﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TechNote24_ChartXScroll_CS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // 차트에서 보여지는 좌표상의 시작값
        private int m_iStart;
        // 하단 X축 Scroll 값
        private int m_iXIncVal;
        // Random 메서드 생성. 무작위값을 대입하여 차트를 그려주기
        private Random m_rdChartVal;
        // 목록의 검색, 정렬 및 조작에 사용할 수 있는 메서드를 제공
        private List<int> m_listChartDatas;

        private void Form1_Load(object sender, EventArgs e)
        {
            // 현재 Form을 MainForm으로 설정합니다.
            smartForm1.MainForm = this;

            // 차트 기본 설정. 폼 디자이너 속성창에서 설정했기 때문에 인자는 생략
            smartDraw1.Chart.SetChartConfig();
            // Chart가 그려질 영역에 격자 이미지를 BackLayer에 그리기
            smartDraw1.BackLayer.ImageDraw(Resource1.ChartGrid, 0, 0);

            smartDraw2.SetFont(16, Color.White);

            for (int i = 0; i < 5; i++)
            {
                smartDraw2.TextOut(5, 210 - (i * 50), ((i + 1) * 50).ToString());
            }
            
            // 차트를 그리기 시작하면 이전 History를 보기 위해 좌측으로 이동해야 하므로 TrackBar의 BarButton을 우측으로 이동
            smartTrackBar1.ValuePercent = 100;

            // 멤버 변수 초기화
            m_iStart = 0;
            m_iXIncVal = 0;
            m_rdChartVal = new Random();
            m_listChartDatas = new List<int>();

            // 하단 X축 SCROLL에 최초 X좌표값 표시
            XScroll_Print();
        }

        // Start 버튼. 차트 그리기 시작
        private void btnDrawStart_Click(object sender, EventArgs e)
        {
            // 클릭 좌표 라인을 그린 뒤 Chart의 Line 색상이 변경되지 않게 하기위한 PenStyle 지정
            smartDraw1.SetPenStyle(Color.White, 1);
            smartTimer1.Start();
        }

        private void smartTimer1_Tick(object sender, EventArgs e)
        {
            int iRVal;

            // 0 ~ 200 범위내의 랜덤한 정수를 반환
            iRVal = m_rdChartVal.Next(0, 200);
            // 차트 데이터가 50일 때마다 한번씩 그려줍니다.
            if (m_listChartDatas.Count % 50 == 0)
            {
                XScroll_Print();
            }
            // List에 해당 값을 대입
            m_listChartDatas.Add(iRVal);
            // 배열이 0부터 시작하므로 -1
            smartDraw1.Chart.PutData(m_listChartDatas[m_listChartDatas.Count - 1]);
        }

        // Stop 버튼. 차트 그리기 정지
        private void btnDrawStop_Click(object sender, EventArgs e)
        {
            smartTimer1.Stop();
        }

        private void smartTrackBar1_Click(object sender, EventArgs e)
        {
            smartTimer1.Stop();

            // List의 Count가 0이 아닌, Count가 1이상인 경우
            if (m_listChartDatas.Count != 0)
            {
                // 기존 그려진 차트 그래프에서 삭제
                smartDraw1.Chart.PutDataAllClear();

                // TrackBar가 100이라고 하면 TrackBar를 클릭할때 해당 위치가 90이면 현재 리스트의 카운트의 90%를 계산해줌
                // ex) 리스트가 100이고 TrackBar의 90을 클릭하면 90/100 = 90%(TrackBar의 지점) x 리스트의 카운트를 하면 총 리스트의 90%의 값을 계산해줌
                m_iStart = (int)((smartTrackBar1.ValuePercent / 100) * m_listChartDatas.Count);

                // iStart값을 현재리스트의 카운트 값보다 적게 만들기위해
                // ex) 현재 리스트 카운트가 100이고 iStart가 50보다 큰 60이라면 리스트 카운트는 110이 되어야 하므로
                if ((m_iStart > m_listChartDatas.Count - (500 / 10)) && m_listChartDatas.Count >= 50)
                {
                    // 현재 리스트 카운트값 - 50을 한값으로 iStart값을 초기화시킴
                    m_iStart = m_listChartDatas.Count - (500 / 10);
                }
                else
                {
                    if (m_listChartDatas.Count < 50)
                    {
                        // 현재 리스트 카운트 값이 50보다 작은 경우에는 -50을 해줄 필요 없음. X좌표의 초기값과 iStart는 동일값
                        m_iStart = 0;
                    }
                }

                // m_iStart 값부터 50개의 값을 그림
                // 새로 그리는 빈도값을 설정. PutData값이 50번마다 그리기
                // smartDraw1.ChartDrawStep = 500(Width) / 10(차트Cursor 간격)
                // iStart 개수부터 iStart+50 개수만큼 반복
                for (int i = m_iStart; i < (m_iStart + (500 / 10)); i++)
                {
                    if (m_listChartDatas.Count > i)
                    {
                        // 리스트의 데이터 개수만큼 출력
                        smartDraw1.Chart.PutData(m_listChartDatas[i]);
                    }
                }
                // 갱신된 X좌표의 값으로 초기화
                XScroll_Print();
                // ChartDraw Step값에 상관없이 즉시 Chart를 다시 그리기
                smartDraw1.Chart.ChartNowDraw();
            }
        }

        private void smartDraw1_MouseDown(object sender, MouseEventArgs e)
        {
            int eX = 0;
            eX = e.X;
            
            // 차트의 특정좌표값을 터치시마다 기존 라인 삭제
            smartDraw1.Erase();
            smartTrackBar1_Click(sender, e);

            // 데이터가 50개 미만 && 사용자 터치 CursorX가 차트 데이터가 표시하는 CursorX보다 큰 경우. 그리는 부분을 제외한 영역을 터치한 경우
            if ((m_listChartDatas.Count < 50) && (m_listChartDatas.Count * 10 < eX))
            {
                smartTimer1.Stop();
            }
            else
            {
                smartDraw1.SetPenStyle(Color.Red, 1);

                // LINE의 X값 불러오기. if X가 10의 단위가 아닌경우. 10으로 끊어줘야함. 
                // 10으로 나눔. 반올림. 10을 곱함
                eX /= 10;
                Math.Ceiling(eX);

                // 현재 그려진 좌표의 개수가 50개를 넘어서면 터치된 좌표값에서 iStart만큼 더해서 표시됨
                if (m_listChartDatas.Count > 50)
                {
                    // 터치한 e.X에 해당하는 m_listChartDatas.Count의 X좌표를 표시
                    // iStart값은 m_listChartDatas.Count에서 -50을 한값
                    labXIndex.Text = ((eX) + m_iStart).ToString();
                    labYIndex.Text = m_listChartDatas[(eX) + m_iStart].ToString();
                }
                else
                {
                    labXIndex.Text = eX.ToString();
                    labYIndex.Text = m_listChartDatas[eX].ToString();
                }
                // LINE 그리기
                smartDraw1.Line(eX * 10, 0, eX * 10, 250);
            }
            btnDrawStart.ButtonUp();
            btnDrawStop.ButtonDown();
        }

        private void XScroll_Print()
        {
            // 50보다 작은 경우 X좌표의 시작점은 0
            if (m_listChartDatas.Count < 50)
            {
                m_iXIncVal = 0;
            }
            // 50보다 큰 경우
            else
            {
                // SmartTimer가 정지라면
                if (smartTimer1.IsStart == false)
                {
                    // SmartTimer가 시작
                    m_iXIncVal = m_iStart;
                }
            }

            // smartDraw1(실제 그래프)이 smartDraw2(Y축 레이블표시)에 비해 우측으로 25정도 위치
            int iXCoord = 30;

            smartDraw2.SetPenStyle(Color.Black, 0);
            smartDraw2.SetBrushStyle(Color.Black);
            smartDraw2.Rectangle(30, 270, 520, 20);
            // X축(Scale) 레이블 초기화  
            smartDraw2.BackLayer.Rectangle(25, 260, 550, 20);

            for (int i = 0; i < 10; i++)
            {
                // 현재 X좌표에서 50만큼 우측으로 표시
                iXCoord += 50;
                // 차트 하단의 X좌표의 +5씩 증가값
                m_iXIncVal += 5;
                // TextOut 메서드로 X축 Scroll에 표시되는 문자 출력
                smartDraw2.TextOut(iXCoord, 270, m_iXIncVal.ToString());
            }
        }

        // 프로그램 종료
        private void btnClose_Click(object sender, EventArgs e)
        {
            smartForm1.Close();
        }
    }
}